@@ -3,13 +3,14 @@ import base64
 import time
 from typing import Any, Dict, Optional, Union
 
-from keylime import config, crypto, json, keylime_logging
+from keylime import agent_util, config, crypto, json, keylime_logging
 from keylime.agentstates import AgentAttestState, AgentAttestStates, TPMClockInfo
 from keylime.common import algorithms
 from keylime.db.verifier_db import VerfierMain
 from keylime.failure import Component, Event, Failure
 from keylime.ima import file_signatures, ima
 from keylime.ima.types import RuntimePolicyType
+from keylime.push_agent_monitor import get_maximum_attestation_interval
 from keylime.tpm import tpm_util
 from keylime.tpm.tpm_main import Tpm
 
@@ -34,7 +35,7 @@ def get_AgentAttestStates() -> AgentAttestStates:
 def process_quote_response(
     agent: Dict[str, Any],
     mb_policy: Optional[str],
-    runtime_policy: RuntimePolicyType,
+    runtime_policy: Optional[RuntimePolicyType],
     json_response: Dict[str, Any],
     agentAttestState: AgentAttestState,
 ) -> Failure:
@@ -82,7 +83,7 @@ def process_quote_response(
 
     # if no public key provided, then ensure we have cached it
     if received_public_key is None:
-        if agent.get("public_key", "") == "" or agent.get("b64_encrypted_V", "") == "":
+        if agent.get("public_key", "") == "" or (agent.get("v") and agent.get("b64_encrypted_V", "") == ""):
             logger.error("agent %s did not provide public key and no key or encrypted_v was cached at CV", agent_id)
             failure.add_event(
                 "no_pubkey",
@@ -177,8 +178,10 @@ def process_quote_response(
     # will be phased out.
     if agent["ima_sign_verification_keys"]:
         verification_key_string = agent["ima_sign_verification_keys"]
-    else:
+    elif runtime_policy is not None:
         verification_key_string = runtime_policy["verification-keys"]
+    else:
+        verification_key_string = ""
 
     tenant_keyring = file_signatures.ImaKeyring.from_string(verification_key_string)
     ima_keyrings.set_tenant_keyring(tenant_keyring)
@@ -259,14 +262,68 @@ def prepare_get_quote(agent: Dict[str, Any]) -> Dict[str, Union[str, int]]:
 
 
 def process_get_status(agent: VerfierMain) -> Dict[str, Any]:
-    has_mb_refstate = 0
+    has_mb_policy = 0
     if agent.mb_policy.mb_policy is not None:
-        has_mb_refstate = 1
+        has_mb_policy = 1
 
     has_runtime_policy = 0
     if agent.ima_policy.generator and agent.ima_policy.generator > ima.RUNTIME_POLICY_GENERATOR.EmptyAllowList:
         has_runtime_policy = 1
 
+    # Get attestation status based on mode (PUSH vs PULL)
+    # Use is_push_mode_agent() to correctly identify PUSH mode agents
+    # (checks both operational_state is None AND ip/port are None)
+    attestation_status = "PENDING"
+
+    if agent_util.is_push_mode_agent(agent):
+        # PUSH mode: determine status based on accept_attestations flag and attestation history
+        # Agent must have successfully attested at least once to show PASS
+        if hasattr(agent, "accept_attestations") and agent.accept_attestations is False:
+            # Timeout has fired - agent is not accepting attestations
+            attestation_status = "FAIL"
+        elif hasattr(agent, "accept_attestations") and agent.accept_attestations is True:
+            # Check if there are recent failures (consecutive_attestation_failures > 0)
+            # This prevents showing PASS when recent attestations failed but timeout hasn't fired yet
+            consecutive_failures = getattr(agent, "consecutive_attestation_failures", None)
+            if consecutive_failures is not None and consecutive_failures > 0:
+                attestation_status = "FAIL"
+            else:
+                # Only show PASS if agent has successfully attested at least once
+                # Use explicit type-safe comparison for attestation_count to satisfy type checkers
+                attestation_count_value = getattr(agent, "attestation_count", None)
+                if attestation_count_value is not None and attestation_count_value > 0:
+                    attestation_status = "PASS"
+                else:
+                    attestation_status = "PENDING"
+        else:
+            attestation_status = "PENDING"
+    else:
+        # PULL mode: determine status based on operational_state
+        from keylime.common import states  # pylint: disable=import-outside-toplevel
+
+        # Active attestation states indicate PASS
+        if agent.operational_state in (
+            states.GET_QUOTE,
+            states.GET_QUOTE_RETRY,
+            states.PROVIDE_V,
+            states.PROVIDE_V_RETRY,
+        ):
+            attestation_status = "PASS"
+        # Failed states indicate FAIL
+        elif agent.operational_state in (states.FAILED, states.INVALID_QUOTE, states.TENANT_FAILED):
+            attestation_status = "FAIL"
+        # Other states (REGISTERED, START, SAVED, TERMINATED) are PENDING
+        else:
+            attestation_status = "PENDING"
+
+    # Get attestation period from config
+    attestation_period_seconds = config.getfloat("verifier", "quote_interval", fallback=2.0)
+    attestation_period = f"{int(attestation_period_seconds)}s"
+
+    # Calculate maximum attestation interval (timeout threshold)
+    maximum_attestation_interval_seconds = get_maximum_attestation_interval(attestation_period_seconds)
+    maximum_attestation_interval = f"{int(maximum_attestation_interval_seconds)}s"
+
     response = {
         "operational_state": agent.operational_state,
         "v": agent.v,
@@ -274,7 +331,7 @@ def process_get_status(agent: VerfierMain) -> Dict[str, Any]:
         "port": agent.port,
         "tpm_policy": agent.tpm_policy,
         "meta_data": agent.meta_data,
-        "has_mb_refstate": has_mb_refstate,
+        "has_mb_refstate": has_mb_policy,
         "has_runtime_policy": has_runtime_policy,
         "accept_tpm_hash_algs": agent.accept_tpm_hash_algs,
         "accept_tpm_encryption_algs": agent.accept_tpm_encryption_algs,
@@ -290,6 +347,9 @@ def process_get_status(agent: VerfierMain) -> Dict[str, Any]:
         "attestation_count": agent.attestation_count,
         "last_received_quote": agent.last_received_quote,
         "last_successful_attestation": agent.last_successful_attestation,
+        "attestation_status": attestation_status,
+        "attestation_period": attestation_period,
+        "maximum_attestation_interval": maximum_attestation_interval,
     }
     return response
 
@@ -377,3 +437,66 @@ def process_verify_identity_quote(
 
     # ok we're done
     return failure
+
+
+def process_verify_attestation(
+    ek: str,
+    ak: str,
+    quote: str,
+    nonce: str,
+    hash_alg: str,
+    tpm_policy: str,
+    runtime_policy: Optional[RuntimePolicyType],
+    mb_policy: str,
+    ima_measurement_list: str,
+    mb_measurement_list: str,
+) -> Failure:
+    """Processes attestation data one time only without requiring registration
+
+    This method is useful to process a node's data without requiring registration. This does not mean we
+    trust the data, only that if it were trusted, what would it's attested state be
+    """
+
+    failure = Failure(Component.QUOTE_VALIDATION)
+
+    # Ensure hash_alg is recognized
+    if not hash_alg or not algorithms.Hash.is_recognized(hash_alg):
+        failure.add_event(
+            "invalid_hash_alg",
+            {"message": f"TPM Quote is using an unrecognized hash algorithm: {hash_alg}", "data": hash_alg},
+            False,
+        )
+        return failure
+
+    assert quote is not None
+    assert ek is not None
+    assert ak is not None
+
+    # check the quote, but policy checks since we only care about identity
+    logger.info("Checking one-shot attestation data")
+    try:
+        quote_failure = get_tpm_instance().check_quote(
+            None,
+            nonce,
+            ek,  # pyright: ignore
+            quote,
+            ak,  # pyright: ignore
+            tpm_policy,
+            ima_measurement_list,
+            runtime_policy,
+            algorithms.Hash(hash_alg),
+            None,  # skip ima_keyrings
+            mb_measurement_list,
+            mb_policy,
+            skip_clock_check=True,
+            skip_pcr_check=False,
+            skip_data_pcr_check=True,
+        )
+        failure.merge(quote_failure)
+    except Exception as e:
+        logger.error("Error verifying quote: %s", str(e))
+        failure.add_event("invalid_quote", {"message": f"Quote validation failed: {e}"}, False)
+        return failure
+
+    # ok we're done
+    return failure
