@@ -3,6 +3,7 @@ import hashlib
 import hmac
 import os
 import secrets
+import uuid
 from typing import Optional, Union
 
 # Crypto implementation using python cryptography package
@@ -15,7 +16,7 @@ from cryptography.hazmat.primitives.asymmetric.rsa import RSAPrivateKey, RSAPubl
 from cryptography.hazmat.primitives.ciphers import AEADEncryptionContext, Cipher, algorithms, modes
 from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
 
-aes_block_size = 16
+AES_BLOCK_SIZE = 16
 
 
 def rsa_import_pubkey(pubkey: Union[str, bytes]) -> RSAPublicKey:
@@ -114,9 +115,11 @@ def rsa_export_privkey(private_key: RSAPrivateKey, password: Optional[str] = Non
     return private_key.private_bytes(
         encoding=serialization.Encoding.PEM,
         format=serialization.PrivateFormat.TraditionalOpenSSL,
-        encryption_algorithm=serialization.BestAvailableEncryption(password.encode("utf-8"))
-        if password
-        else serialization.NoEncryption(),
+        encryption_algorithm=(
+            serialization.BestAvailableEncryption(password.encode("utf-8"))
+            if password
+            else serialization.NoEncryption()
+        ),
     )
 
 
@@ -155,11 +158,11 @@ def generate_random_key(size: int = 32) -> bytes:
 
 
 def strbitxor(a: bytes, b: bytes) -> bytes:
-    a = bytearray(a)
-    b = bytearray(b)
-    retval = bytearray(len(b))
-    for i, _ in enumerate(a):
-        retval[i] = a[i] ^ b[i]
+    a_bytes = bytearray(a)
+    b_bytes = bytearray(b)
+    retval = bytearray(len(b_bytes))
+    for i, _ in enumerate(a_bytes):
+        retval[i] = a_bytes[i] ^ b_bytes[i]
     return bytes(retval)
 
 
@@ -185,7 +188,7 @@ def encrypt(plaintext: Optional[bytes], key: bytes) -> bytes:
     """Encrypt object"""
     if plaintext is None:
         plaintext = b""
-    iv = generate_random_key(aes_block_size)
+    iv = generate_random_key(AES_BLOCK_SIZE)
     encryptor = Cipher(algorithms.AES(key), modes.GCM(iv), backend=default_backend()).encryptor()
     cipher_text = encryptor.update(plaintext) + encryptor.finalize()
     assert isinstance(encryptor, AEADEncryptionContext)
@@ -195,9 +198,150 @@ def encrypt(plaintext: Optional[bytes], key: bytes) -> bytes:
 def decrypt(ciphertext: bytes, key: bytes) -> bytes:
     """Decrypt object"""
     ciphertext = base64.b64decode(ciphertext)
-    iv = ciphertext[:aes_block_size]
-    tag = ciphertext[-aes_block_size:]
-    ciphertext = bytes(ciphertext[aes_block_size:-aes_block_size])
+    iv = ciphertext[:AES_BLOCK_SIZE]
+    tag = ciphertext[-AES_BLOCK_SIZE:]
+    ciphertext = bytes(ciphertext[AES_BLOCK_SIZE:-AES_BLOCK_SIZE])
 
     decryptor = Cipher(algorithms.AES(key), modes.GCM(iv, tag), backend=default_backend()).decryptor()
     return decryptor.update(ciphertext) + decryptor.finalize()
+
+
+def hash_token_for_log(token: str, length: int = 8) -> str:
+    """Hash a token for safe logging.
+
+    Returns the first `length` characters of the SHA256 hash of the token.
+    This allows tokens to be identified in logs for debugging purposes without
+    exposing the actual token value.
+
+    Args:
+        token: The authentication token to hash
+        length: Number of hash characters to return (default: 8)
+
+    Returns:
+        First `length` characters of SHA256 hash, or empty string if token is empty
+    """
+    if not token:
+        return ""
+    return hashlib.sha256(token.encode()).hexdigest()[:length]
+
+
+# PBKDF2 parameters for token hashing (OWASP 2023+ / FIPS-140 compliant)
+_TOKEN_HASH_ITERATIONS = 600_000  # OWASP 2023 minimum for PBKDF2-HMAC-SHA256
+_TOKEN_SALT_LENGTH = 16  # 128 bits, minimum recommended by NIST
+
+
+def generate_token_salt() -> str:
+    """Generate a random salt for token hashing.
+
+    Generates a cryptographically secure random salt as recommended
+    by NIST SP 800-132.
+
+    Returns:
+        Hex-encoded random salt (32 characters for 16 bytes)
+    """
+    return secrets.token_bytes(_TOKEN_SALT_LENGTH).hex()
+
+
+def generate_session_token(session_id: str, secret_bytes: int = 32) -> str:
+    """Generate a session token that embeds the session_id.
+
+    Token format: {session_id}.{random_secret}
+    The session_id prefix allows O(1) lookup by primary key.
+    The random secret provides the entropy for security.
+
+    Args:
+        session_id: The session UUID to embed in the token
+        secret_bytes: Number of random bytes for the secret (default: 32)
+
+    Returns:
+        Token string in format "session_id.secret"
+
+    Raises:
+        ValueError: If session_id is empty
+    """
+    if not session_id:
+        raise ValueError("Session ID cannot be empty")
+    secret = secrets.token_urlsafe(secret_bytes)
+    return f"{session_id}.{secret}"
+
+
+def parse_session_token(token: str) -> tuple[str, str]:
+    """Parse a session token to extract session_id and secret.
+
+    Args:
+        token: The full token in format "session_id.secret"
+
+    Returns:
+        Tuple of (session_id, secret)
+
+    Raises:
+        ValueError: If token format is invalid
+    """
+    if not token or "." not in token:
+        raise ValueError("Invalid token format")
+    session_id, secret = token.split(".", 1)
+    if not session_id or not secret:
+        raise ValueError("Invalid token format: missing session_id or secret")
+    try:
+        uuid.UUID(session_id)
+    except ValueError as exc:
+        raise ValueError("Invalid token format: session_id is not a valid UUID") from exc
+    return session_id, secret
+
+
+def hash_token_for_storage(token: str, salt: str) -> str:
+    """Hash a token for secure storage using PBKDF2 with HMAC-SHA-256.
+
+    Uses PBKDF2 with a per-token salt as recommended by NIST SP 800-132
+    and OWASP 2023. This provides:
+    - Protection against rainbow table attacks (unique salt per token)
+    - Computational cost for brute-force attempts (600k iterations per OWASP 2023)
+    - FIPS-140 compliant key derivation
+
+    Args:
+        token: The authentication token to hash
+        salt: Hex-encoded salt (from generate_token_salt())
+
+    Returns:
+        Hex-encoded PBKDF2 hash (64 characters for 32 bytes)
+
+    Raises:
+        ValueError: If token or salt is empty/invalid
+    """
+    if not token:
+        raise ValueError("Token cannot be empty for storage")
+    if not salt or len(salt) != _TOKEN_SALT_LENGTH * 2:
+        raise ValueError(f"Salt must be {_TOKEN_SALT_LENGTH * 2} hex characters")
+
+    token_kdf = PBKDF2HMAC(
+        algorithm=hashes.SHA256(),
+        length=32,
+        salt=bytes.fromhex(salt),
+        iterations=_TOKEN_HASH_ITERATIONS,
+        backend=default_backend(),
+    )
+    return token_kdf.derive(token.encode()).hex()
+
+
+def verify_token_hash(token: str, salt: str, stored_hash: str) -> bool:
+    """Verify a token against a stored PBKDF2 hash.
+
+    Uses constant-time comparison to prevent timing attacks.
+
+    Args:
+        token: The plaintext token to verify
+        salt: Hex-encoded salt used when the hash was created
+        stored_hash: The hex-encoded PBKDF2 hash from the database
+
+    Returns:
+        True if the token matches the stored hash, False otherwise
+    """
+    if not token or not salt or not stored_hash:
+        return False
+
+    try:
+        computed_hash = hash_token_for_storage(token, salt)
+        # Use constant-time comparison to prevent timing attacks
+        return hmac.compare_digest(computed_hash, stored_hash)
+    except Exception:
+        return False
