@@ -1,6 +1,6 @@
-"""Tagging of failure events that might cause revocation in Keylime.
+"""Tagging of failure events that might cause revocation in Keylime."""
 
-"""
+# pylint: disable=R0401
 
 import enum
 import functools
@@ -49,6 +49,7 @@ class Component(enum.Enum):
     MEASURED_BOOT = "measured_boot"
     IMA = "ima"
     INTERNAL = "internal"
+    TEE = "tee_attestation"
     DEFAULT = "default"
 
 
