@@ -4,6 +4,7 @@ import json
 import struct
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+import jsonschema
 from cryptography import x509
 from cryptography.exceptions import InvalidSignature, UnsupportedAlgorithm
 from cryptography.hazmat import backends
@@ -22,6 +23,22 @@ logger = keylime_logging.init_logging("file_signatures")
 
 SupportedKeyTypes = Union[RSAPublicKey, EllipticCurvePublicKey]
 
+IMA_KEYRING_JSON_SCHEMA = {
+    "type": "object",
+    "required": ["keyids", "pubkeys"],
+    "properties": {
+        "meta": {
+            "type": "object",
+            "required": ["version"],
+            "properties": {
+                "version": {"type": "integer", "minimum": 1},
+            },
+        },
+        "keyids": {"type": "array", "items": {"type": "integer"}},
+        "pubkeys": {"type": "array", "items": {"type": "string"}},
+    },
+}
+
 """
 Tools for IMA file signature verification
 """
@@ -196,6 +213,12 @@ class ImaKeyring:
         if not isinstance(obj, dict):
             return None
 
+        try:
+            jsonschema.validate(obj, IMA_KEYRING_JSON_SCHEMA)
+        except Exception as e:
+            logger.error("JSON from string is not a valid IMA Keyring : %s", e)
+            return None
+
         keyids = obj.get("keyids", [])
 
         for der_key, keyidv2 in ImaKeyring._base64_to_der_keylist(obj["pubkeys"], keyids):
@@ -244,6 +267,13 @@ class ImaKeyrings:
         else:
             self.keyrings.pop("tenant_keyring", None)
 
+    def update(self, other):  # type: ignore[no-untyped-def]
+        if not isinstance(other, ImaKeyrings):
+            raise TypeError(f"cannot update ImaKeyrings object from object of type '{other.__class__.__name__}'")
+
+        for name, keyring in other.keyrings.items():
+            self.keyrings[name] = keyring
+
     def get_tenant_keyring(self) -> Optional[ImaKeyring]:
         """Get the tenant keyring."""
         return self.keyrings.get("tenant_keyring")
