@@ -1,6 +1,4 @@
-"""Database migration
-
-"""
+"""Database migration"""
 
 import logging
 import re
@@ -8,7 +6,7 @@ import sys
 
 from alembic import context
 
-from keylime.db.keylime_db import DBEngineManager
+from keylime.db.keylime_db import make_engine
 from keylime.db.registrar_db import Base as RegistrarBase
 from keylime.db.verifier_db import Base as VerifierBase
 
@@ -74,7 +72,7 @@ def run_migrations_offline():
         logger.info("Writing output to %s", file_)
 
         with open(file_, "w", encoding="utf-8") as buffer:
-            engine = DBEngineManager().make_engine(name)
+            engine = make_engine(name)
             connection = engine.connect()
             context.configure(
                 connection=connection,
@@ -83,6 +81,9 @@ def run_migrations_offline():
                 literal_binds=True,
                 dialect_opts={"paramstyle": "named"},
                 version_table="alembic_version_" + name,
+                # Generate migrations using batch mode for SQLite
+                # See https://alembic.sqlalchemy.org/en/latest/batch.html#batch-mode-with-autogenerate
+                render_as_batch=True,
             )
             with context.begin_transaction():
                 context.run_migrations(engine_name=name)
@@ -102,7 +103,7 @@ def run_migrations_online():
     engines = {}
     for name in re.split(r",\s*", db_names):
         engines[name] = rec = {}
-        rec["engine"] = DBEngineManager().make_engine(name)
+        rec["engine"] = make_engine(name)
 
     for name, rec in engines.items():
         engine = rec["engine"]
