@@ -5,13 +5,13 @@ Revises: f35cdd35eb83
 Create Date: 2021-08-20 12:42:30.427138
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = '257fe0f0c039'
-down_revision = 'f35cdd35eb83'
+revision = "257fe0f0c039"
+down_revision = "f35cdd35eb83"
 branch_labels = None
 depends_on = None
 
@@ -24,9 +24,6 @@ def downgrade(engine_name):
     globals()[f"downgrade_{engine_name}"]()
 
 
-
-
-
 def upgrade_registrar():
     pass
 
@@ -36,10 +33,10 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    op.add_column('verifiermain', sa.Column('severity_level', sa.Integer))
-    op.add_column('verifiermain', sa.Column('last_event_id', sa.String(100)))
+    op.add_column("verifiermain", sa.Column("severity_level", sa.Integer))
+    op.add_column("verifiermain", sa.Column("last_event_id", sa.String(100)))
 
 
 def downgrade_cloud_verifier():
-    op.drop_column('verifiermain', 'severity_level')
-    op.drop_column('verifiermain', sa.Column('last_event_id', sa.Integer))
+    op.drop_column("verifiermain", "severity_level")
+    op.drop_column("verifiermain", sa.Column("last_event_id", sa.Integer))
