@@ -5,13 +5,13 @@ Revises: eeb702f77d7d
 Create Date: 2021-01-14 10:50:56.275257
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = '8da20383f6e1'
-down_revision = 'eeb702f77d7d'
+revision = "8da20383f6e1"
+down_revision = "eeb702f77d7d"
 branch_labels = None
 depends_on = None
 
@@ -33,9 +33,10 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    with op.batch_alter_table('verifiermain') as batch_op:
-        batch_op.alter_column('ip', existing_type=sa.String(length=15),
-                              type_=sa.String(length=255), existing_nullable=True)
+    with op.batch_alter_table("verifiermain") as batch_op:
+        batch_op.alter_column(
+            "ip", existing_type=sa.String(length=15), type_=sa.String(length=255), existing_nullable=True
+        )
 
 
 def downgrade_cloud_verifier():
