@@ -5,13 +5,13 @@ Revises: eeb702f77d7d
 Create Date: 2020-11-18 10:37:58.244212
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = 'a7a64155ab3a'
-down_revision = '8da20383f6e1'
+revision = "a7a64155ab3a"
+down_revision = "8da20383f6e1"
 branch_labels = None
 depends_on = None
 
@@ -33,8 +33,8 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    op.add_column('verifiermain', sa.Column('ima_sign_verification_keys', sa.String(1000)))
+    op.add_column("verifiermain", sa.Column("ima_sign_verification_keys", sa.String(1000)))
 
 
 def downgrade_cloud_verifier():
-    op.drop_column('verifiermain', 'ima_sign_verification_keys')
+    op.drop_column("verifiermain", "ima_sign_verification_keys")
