@@ -5,13 +5,13 @@ Revises: cc2630851a1f
 Create Date: 2021-03-03 11:49:00.860132
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = 'ae898986c6e9'
-down_revision = 'cc2630851a1f'
+revision = "ae898986c6e9"
+down_revision = "cc2630851a1f"
 branch_labels = None
 depends_on = None
 
@@ -24,9 +24,6 @@ def downgrade(engine_name):
     globals()[f"downgrade_{engine_name}"]()
 
 
-
-
-
 def upgrade_registrar():
     pass
 
@@ -36,8 +33,8 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    op.add_column('verifiermain', sa.Column('mb_refstate', sa.String(1000)))
+    op.add_column("verifiermain", sa.Column("mb_refstate", sa.String(1000)))
 
 
 def downgrade_cloud_verifier():
-    op.drop_column('verifiermain', 'mb_refstate')
+    op.drop_column("verifiermain", "mb_refstate")
