@@ -5,13 +5,13 @@ Revises: eb869a77abd1
 Create Date: 2021-05-06 18:06:47.637118
 
 """
-from alembic import op
-import sqlalchemy as sa
 
+import sqlalchemy as sa
+from alembic import op
 
 # revision identifiers, used by Alembic.
-revision = 'b4d024197413'
-down_revision = 'eb869a77abd1'
+revision = "b4d024197413"
+down_revision = "eb869a77abd1"
 branch_labels = None
 depends_on = None
 
@@ -24,9 +24,6 @@ def downgrade(engine_name):
     globals()[f"downgrade_{engine_name}"]()
 
 
-
-
-
 def upgrade_registrar():
     pass
 
@@ -36,12 +33,12 @@ def downgrade_registrar():
 
 
 def upgrade_cloud_verifier():
-    op.add_column('verifiermain', sa.Column('verifier_id', sa.String(100)))
-    op.add_column('verifiermain', sa.Column('verifier_ip', sa.String(20)))
-    op.add_column('verifiermain', sa.Column('verifier_port', sa.Integer))
+    op.add_column("verifiermain", sa.Column("verifier_id", sa.String(100)))
+    op.add_column("verifiermain", sa.Column("verifier_ip", sa.String(20)))
+    op.add_column("verifiermain", sa.Column("verifier_port", sa.Integer))
 
 
 def downgrade_cloud_verifier():
-    op.drop_column('verifiermain', 'verifier_id')
-    op.drop_column('verifiermain', 'verifier_ip')
-    op.drop_column('verifiermain', 'verifier_port')
+    op.drop_column("verifiermain", "verifier_id")
+    op.drop_column("verifiermain", "verifier_ip")
+    op.drop_column("verifiermain", "verifier_port")
