@@ -1,3 +1,4 @@
+import re
 import ssl
 from typing import Any, Dict, Optional
 
@@ -15,6 +16,10 @@ class RequestsClient:
         ignore_hostname: bool = True,
         **kwargs: Any,
     ) -> None:
+        # Remove eventual "http?://" from the base url
+        if base_url.startswith("http"):
+            base_url = re.sub(r"https?://", "", base_url)
+
         if tls_enabled:
             self.base_url = f"https://{base_url}"
         else:
@@ -35,7 +40,10 @@ class RequestsClient:
         return self
 
     def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
-        pass
+        try:
+            self.session.close()
+        except Exception:
+            pass
 
     def request(self, method: str, url: str, **kwargs: Any) -> requests.Response:
         return self.session.request(method, self.base_url + url, **kwargs)
