@@ -36,6 +36,7 @@
 #  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #
 
+import base64
 import binascii
 import os
 import string
@@ -49,14 +50,53 @@ from cryptography.hazmat.primitives.asymmetric.ec import EllipticCurvePublicKey
 from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey
 from cryptography.hazmat.primitives.asymmetric.utils import Prehashed
 from cryptography.hazmat.primitives.ciphers import Cipher, modes
+from cryptography.hazmat.primitives.kdf.concatkdf import ConcatKDFHash
+from cryptography.hazmat.primitives.kdf.kbkdf import KBKDFHMAC, CounterLocation, Mode
 
 from keylime import config, crypto, json, keylime_logging
-from keylime.tpm import ec_crypto_helper, tpm2_objects
+from keylime.tpm import tpm2_objects
 
 logger = keylime_logging.init_logging("tpm_util")
 
 SupportedKeyTypes = Union[RSAPublicKey, EllipticCurvePublicKey]
 
+# ECC signature parsing constants.
+# Raw signature sizes for different ECC curves (r||s concatenated format).
+# r and s are the two mathematical components of an ECDSA signature.
+ECC_SECP192R1_SIGNATURE_SIZE = 48  # 24 bytes each for r,s.
+ECC_SECP224R1_SIGNATURE_SIZE = 56  # 28 bytes each for r,s.
+ECC_SECP256R1_SIGNATURE_SIZE = 64  # 32 bytes each for r,s.
+ECC_SECP384R1_SIGNATURE_SIZE = 96  # 48 bytes each for r,s.
+ECC_SECP521R1_SIGNATURE_SIZE = 132  # 66 bytes each for r,s.
+
+# TPM2B_ECDSA_SIGNATURE format constants.
+TPM2B_SIZE_FIELD_LENGTH = 2  # 2 bytes for size field.
+TPM2B_MIN_HEADER_SIZE = 4  # Minimum: 2 bytes r_size + 2 bytes s_size.
+
+# DER encoding constants.
+DER_SEQUENCE_TAG = 0x30
+
+# Signature blob header offset (skip alg, hash_alg, sig_size headers).
+SIGNATURE_BLOB_HEADER_SIZE = 6
+
+# ECC curve to signature size mapping for raw r||s format.
+ECC_RAW_SIGNATURE_SIZES = {
+    "secp192r1": ECC_SECP192R1_SIGNATURE_SIZE,
+    "secp224r1": ECC_SECP224R1_SIGNATURE_SIZE,
+    "secp256r1": ECC_SECP256R1_SIGNATURE_SIZE,
+    "secp384r1": ECC_SECP384R1_SIGNATURE_SIZE,
+    "secp521r1": ECC_SECP521R1_SIGNATURE_SIZE,
+}
+
+# ECC curve coordinate size ranges (min, max) for validation.
+ECC_COORDINATE_SIZE_RANGES = {
+    "secp192r1": (1, 24),  # 192 bits = 24 bytes max
+    "secp224r1": (1, 28),  # 224 bits = 28 bytes max
+    "secp256r1": (1, 32),  # 256 bits = 32 bytes max
+    "secp384r1": (1, 48),  # 384 bits = 48 bytes max
+    "secp521r1": (1, 66),  # 521 bits = 66 bytes max (521 bits, not 512)
+}
+
 
 def verify(
     pubkey: SupportedKeyTypes,
@@ -104,17 +144,59 @@ def der_len(encoded_int_len: int) -> bytes:
     return bytes((0x80 | len(bin_str),)) + bin_str
 
 
-def ecdsa_der_from_tpm(sigblob: bytes) -> bytes:
-    _, _, sig_size_r = struct.unpack_from(">HHH", sigblob, 0)
-    sig_r = sigblob[6 : 6 + sig_size_r]
-    encoded_sig_r = der_int(sig_r)
-    sigblob = sigblob[6 + sig_size_r :]
-    sig_size_s = struct.unpack_from(">H", sigblob, 0)[0]
-    sig_s = sigblob[2 : 2 + sig_size_s]
-    encoded_sig_s = der_int(sig_s)
-    total_size = len(encoded_sig_r) + len(encoded_sig_s)
-    der_sig = bytes.fromhex(f"30{total_size:x}") + encoded_sig_r + encoded_sig_s
-    return der_sig
+def ecdsa_der_from_tpm(sigblob: bytes, pubkey: EllipticCurvePublicKey) -> bytes:
+    """Convert ECC signature from TPM format to DER format for cryptographic verification.
+
+    This function handles TSS ESAPI signature format where the signature header's
+    sig_size field contains the size of the r component, followed by the s component
+    with its own size header.
+
+    Parameters
+    ----------
+    sigblob: TPM signature blob containing signature headers and signature data
+    pubkey: ECC public key to determine expected signature size
+
+    Returns
+    -------
+    DER-encoded ECDSA signature suitable for cryptographic library verification
+
+    Raises
+    ------
+    ValueError: If signature format cannot be parsed or is invalid
+    """
+    # Extract signature header information.
+    _sig_alg, _hash_alg, sig_size_r = struct.unpack_from(">HHH", sigblob, 0)
+
+    # Extract the r component (size is in sig_size_r field).
+    sig_r = sigblob[SIGNATURE_BLOB_HEADER_SIZE : SIGNATURE_BLOB_HEADER_SIZE + sig_size_r]
+
+    # The s component follows immediately after r, with its own size header.
+    s_offset = SIGNATURE_BLOB_HEADER_SIZE + sig_size_r
+    if s_offset + 2 <= len(sigblob):
+        sig_size_s = struct.unpack_from(">H", sigblob, s_offset)[0]
+        s_start = s_offset + 2
+        if s_start + sig_size_s <= len(sigblob):
+            sig_s = sigblob[s_start : s_start + sig_size_s]
+
+            # Validate coordinate sizes against curve requirements.
+            curve_name = pubkey.curve.name
+            coordinate_range = ECC_COORDINATE_SIZE_RANGES.get(curve_name)
+            if coordinate_range:
+                min_size, max_size = coordinate_range
+                if not min_size <= len(sig_r) <= max_size:
+                    raise ValueError(f"Invalid r coordinate size {len(sig_r)} for curve {curve_name}")
+                if not min_size <= len(sig_s) <= max_size:
+                    raise ValueError(f"Invalid s coordinate size {len(sig_s)} for curve {curve_name}")
+
+            # Convert to DER format.
+            encoded_sig_r = der_int(sig_r)
+            encoded_sig_s = der_int(sig_s)
+            total_size = len(encoded_sig_r) + len(encoded_sig_s)
+            der_length = der_len(total_size)
+            der_sig = bytes([DER_SEQUENCE_TAG]) + der_length + encoded_sig_r + encoded_sig_s
+            return der_sig
+
+    raise ValueError("Unable to parse ECC signature from TPM format")
 
 
 def __get_pcrs_from_blob(pcrblob: bytes) -> Tuple[int, Dict[int, int], List[bytes]]:
@@ -221,9 +303,7 @@ def checkquote(
     pcrblob: The state of the PCRs that were quoted; Intel tpm2-tools specific format
     exp_hash_alg: The hash that was expected to have been used for quoting
     """
-    sig_alg, hash_alg, sig_size = struct.unpack_from(">HHH", sigblob, 0)
-
-    (signature,) = struct.unpack_from(f"{sig_size}s", sigblob, 6)
+    sig_alg, hash_alg = struct.unpack_from(">HH", sigblob, 0)
 
     pubkey = serialization.load_pem_public_key(aikblob, backend=backends.default_backend())
     if not isinstance(pubkey, (RSAPublicKey, EllipticCurvePublicKey)):
@@ -234,6 +314,16 @@ def checkquote(
     if isinstance(pubkey, EllipticCurvePublicKey) and sig_alg not in [tpm2_objects.TPM_ALG_ECDSA]:
         raise ValueError(f"Unsupported quote signature algorithm '{sig_alg:#x}' for EC keys")
 
+    if sig_alg in [tpm2_objects.TPM_ALG_RSASSA]:
+        (sig_size,) = struct.unpack_from(">H", sigblob, 4)
+        (signature,) = struct.unpack_from(f"{sig_size}s", sigblob, 6)
+    elif sig_alg in [tpm2_objects.TPM_ALG_ECDSA]:
+        if not isinstance(pubkey, EllipticCurvePublicKey):
+            raise ValueError(f"ECDSA signature algorithm requires EllipticCurvePublicKey, got {type(pubkey)}")
+        signature = ecdsa_der_from_tpm(sigblob, pubkey)
+    else:
+        raise ValueError(f"Unsupported quote signature algorithm '{sig_alg:#x}'")
+
     hashfunc = tpm2_objects.HASH_FUNCS.get(hash_alg)
     if not hashfunc:
         raise ValueError(f"Unsupported hash with id {hash_alg:#x} in signature blob")
@@ -249,7 +339,31 @@ def checkquote(
     # Check that reported nonce is expected one
     retDict = tpm2_objects.unmarshal_tpms_attest(quoteblob)
     extradata = retDict["extraData"]
-    if extradata.decode("utf-8") != nonce:
+
+    # The nonce parameter might be base64-encoded (push mode) or raw bytes/string (pull mode)
+    # The extradata from TPM is always raw bytes
+    # Auto-detect the encoding by trying both interpretations
+
+    nonce_matched = False
+
+    # Try raw interpretation first (pull mode - most common)
+    if isinstance(nonce, str):
+        if extradata == nonce.encode("utf-8"):
+            nonce_matched = True
+    elif isinstance(nonce, bytes):
+        if extradata == nonce:
+            nonce_matched = True
+
+    # If raw interpretation didn't match, try base64 decoding (push mode)
+    if not nonce_matched and isinstance(nonce, str):
+        try:
+            decoded = base64.b64decode(nonce)
+            if extradata == decoded:
+                nonce_matched = True
+        except Exception:
+            pass
+
+    if not nonce_matched:
         raise Exception("The nonce from the attestation differs from the expected nonce")
 
     # Check that correct quote_digest was used which is equivalent to hash(quoteblob)
@@ -306,15 +420,18 @@ def crypt_secret_encrypt_ecc(public_key: EllipticCurvePublicKey, hashfunc: hashe
     my_public_key = my_private_key.public_key()
     point = tpm2_objects.tpms_ecc_point_marshal(my_public_key)
 
-    ecc_secret_x = ec_crypto_helper.EcCryptoHelper.get_instance().point_multiply_x(public_key, my_private_key)
+    ecc_secret_x = my_private_key.exchange(ec.ECDH(), public_key)
 
     digest_size = hashfunc.digest_size
 
+    # Use fixed coordinate size for consistent marshaling
+    coord_size = (public_key.curve.key_size + 7) // 8
+
     x = my_public_key.public_numbers().x
-    party_x = x.to_bytes((x.bit_length() + 7) >> 3, "big")
+    party_x = x.to_bytes(coord_size, "big")
 
     x = public_key.public_numbers().x
-    party_y = x.to_bytes((x.bit_length() + 7) >> 3, "big")
+    party_y = x.to_bytes(coord_size, "big")
 
     data = crypt_kdfe(hashfunc, ecc_secret_x, "IDENTITY", party_x, party_y, digest_size << 3)
 
@@ -449,27 +566,20 @@ def crypt_kdfa(
     size_in_bits: how many bits of random data to generate
     """
     size_in_bytes = (size_in_bits + 7) >> 3
-    label_bytes = label_to_bytes(label)
     context = context_u + context_v
 
-    ctr = 0
-    result = b""
-
-    size = size_in_bytes
-
-    while size > 0:
-        h = hmac.HMAC(key, hashfunc)
-        ctr += 1
-        h.update(struct.pack(">I", ctr))
-        h.update(label_bytes)
-        if context:
-            h.update(context)
-        h.update(struct.pack(">I", size_in_bits))
-        result += h.finalize()
-
-        size -= hashfunc.digest_size
-
-    return result[:size_in_bytes]
+    kdf = KBKDFHMAC(
+        algorithm=hashfunc,
+        mode=Mode.CounterMode,
+        length=size_in_bytes,
+        rlen=4,
+        llen=4,
+        location=CounterLocation.BeforeFixed,
+        label=bytes(label, "UTF-8"),
+        context=context,
+        fixed=None,
+    )
+    return kdf.derive(key)
 
 
 def crypt_kdfe(
@@ -493,27 +603,14 @@ def crypt_kdfe(
     """
     size_in_bytes = (size_in_bits + 7) >> 3
     label_bytes = label_to_bytes(label)
-    party = party_x + party_y
+    otherinfo = label_bytes + party_x + party_y
 
-    ctr = 0
-    result = b""
-
-    size = size_in_bytes
-
-    while size > 0:
-        digest = hashes.Hash(hashfunc, backend=backends.default_backend())
-        ctr += 1
-        digest.update(struct.pack(">I", ctr))
-        if secret_x:
-            digest.update(secret_x)
-        digest.update(label_bytes)
-        if party:
-            digest.update(party)
-        result += digest.finalize()
-
-        size -= hashfunc.digest_size
-
-    return result[:size_in_bytes]
+    kdf = ConcatKDFHash(
+        algorithm=hashfunc,
+        length=size_in_bytes,
+        otherinfo=otherinfo,
+    )
+    return kdf.derive(secret_x)
 
 
 def crypt_hash(data: bytes, hash_alg: bytes) -> Tuple[bytes, hashes.HashAlgorithm]:
