@@ -1,8 +1,3 @@
-"""
-SPDX-License-Identifier: Apache-2.0
-Copyright 2022 Red Hat, Inc.
-"""
-
 import unittest
 from pathlib import Path
 
@@ -10,7 +5,8 @@ from keylime import signing
 
 # Useful constants for the test
 PACKAGE_ROOT = Path(__file__).parents[1]
-TEST_FILES = (f"{PACKAGE_ROOT}/test-data/files")
+TEST_FILES = f"{PACKAGE_ROOT}/test-data/files"
+
 
 class TestSigning(unittest.TestCase):
     def test_sign_gpg(self):
@@ -19,7 +15,7 @@ class TestSigning(unittest.TestCase):
                 f"{TEST_FILES}/allowlist-pgp-key.pgp",
                 f"{TEST_FILES}/allowlist.json",
                 f"{TEST_FILES}/allowlist-pgp-sig.sig",
-                "Testing Allowlist"
+                "Testing Allowlist",
             )
         except Exception as e:
             self.fail(f"Signing raised exception: {e}!")
@@ -30,7 +26,7 @@ class TestSigning(unittest.TestCase):
                 f"{TEST_FILES}/allowlist-ec-key.pem",
                 f"{TEST_FILES}/allowlist.json",
                 f"{TEST_FILES}/allowlist-ec-sig.bin",
-                "Testing Allowlist"
+                "Testing Allowlist",
             )
         except Exception as e:
             self.fail(f"Signing raised exception: {e}!")
@@ -41,7 +37,7 @@ class TestSigning(unittest.TestCase):
                 f"{TEST_FILES}/allowlist-pgp-key.pgp",
                 f"{TEST_FILES}/allowlist.json",
                 f"{TEST_FILES}/allowlist-invalid-sig.sig",
-                "Testing Allowlist"
+                "Testing Allowlist",
             )
             self.fail("Signing passed with invalid signature!")
         except Exception:
